import { Module } from '@nestjs/common';
import { CommonService } from './common.service';
import { DatabaseModule } from './database/database.module';
import { HelperModule } from './helper/helper.module';
import { AuthModule } from './auth/auth.module';
import { ConfigModule } from '@nestjs/config';
import config from './config';
import { EmailService } from './util/email.service';
import { ErrorMessageService } from 'src/error-message/error-message.service';
import { TuyaService } from './integrations/tuya/services/tuya.service';
import { SceneDeviceRepository } from './modules/scene-device/repositories';
import { SpaceRepository } from './modules/space';
import {
  SpaceModelRepository,
  SubspaceModelRepository,
} from './modules/space-model';
import { SubspaceRepository } from './modules/space/repositories/subspace.repository';
@Module({
  providers: [
    CommonService,
    EmailService,
    ErrorMessageService,
    TuyaService,
    SceneDeviceRepository,
    SpaceRepository,
    SubspaceRepository,
    SubspaceModelRepository,
    SpaceModelRepository,
  ],
  exports: [
    CommonService,
    TuyaService,
    HelperModule,
    AuthModule,
    EmailService,
    ErrorMessageService,
    SceneDeviceRepository,
    SpaceRepository,
    SubspaceRepository,
    SubspaceModelRepository,
    SpaceModelRepository,
  ],
  imports: [
    ConfigModule.forRoot({
      load: config,
      isGlobal: true,
    }),
    DatabaseModule,
    HelperModule,
    AuthModule,
  ],
})
export class CommonModule {}
