import { Controller, Post, Param } from '@nestjs/common';
import { ApiBearerAuth, ApiOperation, ApiTags } from '@nestjs/swagger';
import { AddDeviceStatusDto } from '../dtos/add.devices-status.dto';
import { DeviceStatusFirebaseService } from '../services/devices-status.service';
import { EnableDisableStatusEnum } from '@app/common/constants/days.enum';
import { ControllerRoute } from '@app/common/constants/controller-route';

@ApiTags('Device Status Firebase Module')
@Controller({
  version: EnableDisableStatusEnum.ENABLED,
  path: ControllerRoute.DEVICE_STATUS_FIREBASE.ROUTE,
})
export class DeviceStatusFirebaseController {
  constructor(
    private readonly deviceStatusFirebaseService: DeviceStatusFirebaseService,
  ) {}

  @ApiBearerAuth()
  @Post(':deviceTuyaUuid')
  @ApiOperation({
    summary:
      ControllerRoute.DEVICE_STATUS_FIREBASE.ACTIONS.ADD_DEVICE_STATUS_SUMMARY,
    description:
      ControllerRoute.DEVICE_STATUS_FIREBASE.ACTIONS
        .ADD_DEVICE_STATUS_DESCRIPTION,
  })
  async addDeviceStatus(
    @Param('deviceTuyaUuid') deviceTuyaUuid: string,
  ): Promise<AddDeviceStatusDto> {
    return this.deviceStatusFirebaseService.addDeviceStatusByDeviceUuid(
      deviceTuyaUuid,
    );
  }
}
