import { HttpStatus, Injectable, NotFoundException } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';

import * as crypto from 'crypto';
import { RegisterClientDto } from '../dtos/register-client.dto';
import { ClientEntity } from '@app/common/modules/client/entities';
import { ClientTokenDto } from '../dtos/token-client.dto';
import { AuthService } from '@app/common/auth/services/auth.service';
import { SuccessResponseDto } from '@app/common/dto/success.response.dto';

@Injectable()
export class ClientService {
  constructor(
    @InjectRepository(ClientEntity)
    private clientRepository: Repository<ClientEntity>,
    private readonly authService: AuthService,
  ) {}
  async loginWithClientCredentials(dto: ClientTokenDto) {
    const client = await this.validateClient(dto.clientId, dto.clientSecret);
    const payload = {
      client: {
        clientId: client.clientId,
        uuid: client.uuid,
        scopes: client.scopes,
      },
    };
    const tokens = await this.authService.getTokens(payload, false, '5m');
    return new SuccessResponseDto({
      message: `Client logged in successfully`,
      data: tokens,
      statusCode: HttpStatus.CREATED,
    });
  }
  async registerClient(dto: RegisterClientDto) {
    const clientId = crypto.randomBytes(16).toString('hex');
    const clientSecret = crypto.randomBytes(32).toString('hex');

    const client = this.clientRepository.create({
      name: dto.name,
      clientId,
      clientSecret,
      redirectUri: dto.redirectUri,
      scopes: dto.scopes,
    });

    await this.clientRepository.save(client);

    return new SuccessResponseDto({
      message: `Client registered successfully`,
      data: { clientId, clientSecret },
      statusCode: HttpStatus.CREATED,
    });
  }

  async validateClient(
    clientId: string,
    clientSecret: string,
  ): Promise<ClientEntity | null> {
    const client = await this.clientRepository.findOne({
      where: { clientId, clientSecret },
    });
    if (!client) {
      throw new NotFoundException('Invalid client credentials');
    }
    return client;
  }
}
