import { ApiProperty } from '@nestjs/swagger';
import { IsNotEmpty, IsOptional, IsString, IsUUID } from 'class-validator';

export class AddCommunityDto {
  @ApiProperty({
    description: 'The name of the community',
    example: 'Community A',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public name: string;

  @ApiProperty({
    description: 'A description of the community',
    example: 'This is a community for developers.',
    required: false,
  })
  @IsString()
  @IsOptional()
  public description?: string;

  constructor(dto: Partial<AddCommunityDto>) {
    Object.assign(this, dto);
  }
}

export class AddUserCommunityDto {
  @ApiProperty({
    description: 'communityUuid',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public communityUuid: string;
  @ApiProperty({
    description: 'userUuid',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public userUuid: string;
  constructor(dto: Partial<AddUserCommunityDto>) {
    Object.assign(this, dto);
  }
}
