import { ApiProperty } from '@nestjs/swagger';
import {
  IsNotEmpty,
  IsString,
  IsArray,
  ValidateNested,
  IsEnum,
  Length,
  IsOptional,
} from 'class-validator';
import { Type } from 'class-transformer';
import { WorkingDays } from '@app/common/constants/working-days';

class ScheduleDto {
  @ApiProperty({
    description: 'effectiveTime',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public effectiveTime: string;

  @ApiProperty({
    description: 'invalidTime',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public invalidTime: string;

  @ApiProperty({
    description: 'workingDay',
    enum: WorkingDays,
    isArray: true,
    required: true,
  })
  @IsArray()
  @IsEnum(WorkingDays, { each: true })
  @IsNotEmpty()
  public workingDay: WorkingDays[];
}

export class AddDoorLockOnlineDto {
  @ApiProperty({
    description: 'name',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public name: string;
  @ApiProperty({
    description: 'password',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  @Length(7, 7)
  public password: string;

  @ApiProperty({
    description: 'effectiveTime',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public effectiveTime: string;

  @ApiProperty({
    description: 'invalidTime',
    required: true,
  })
  @IsString()
  @IsNotEmpty()
  public invalidTime: string;

  @ApiProperty({
    description: 'scheduleList',
    type: [ScheduleDto],
    required: false,
  })
  @IsArray()
  @ValidateNested({ each: true })
  @IsOptional()
  @Type(() => ScheduleDto)
  public scheduleList: ScheduleDto[];
}
