import { SpaceRepository } from '@app/common/modules/space/repositories';
import {
  Injectable,
  CanActivate,
  HttpStatus,
  ExecutionContext,
  BadRequestException,
} from '@nestjs/common';

@Injectable()
export class CheckFloorTypeGuard implements CanActivate {
  constructor(private readonly spaceRepository: SpaceRepository) {}

  async canActivate(context: ExecutionContext): Promise<boolean> {
    const req = context.switchToHttp().getRequest();
    try {
      const { unitName, floorUuid } = req.body;

      if (!unitName) {
        throw new BadRequestException('unitName is required');
      }

      if (!floorUuid) {
        throw new BadRequestException('floorUuid is required');
      }

      await this.checkFloorIsFloorType(floorUuid);

      return true;
    } catch (error) {
      this.handleGuardError(error, context);
      return false;
    }
  }

  async checkFloorIsFloorType(floorUuid: string) {
    const floorData = await this.spaceRepository.findOne({
      where: { uuid: floorUuid },
    });

    if (!floorData) {
      throw new BadRequestException('Invalid floor UUID');
    }
  }

  private handleGuardError(error: Error, context: ExecutionContext) {
    const response = context.switchToHttp().getResponse();
    console.error(error);

    if (error instanceof BadRequestException) {
      response
        .status(HttpStatus.BAD_REQUEST)
        .json({ statusCode: HttpStatus.BAD_REQUEST, message: error.message });
    } else {
      response.status(HttpStatus.NOT_FOUND).json({
        statusCode: HttpStatus.NOT_FOUND,
        message: 'Floor not found',
      });
    }
  }
}
