import { Body, Controller, Get, Post, UseGuards } from '@nestjs/common';
import { ApiTags, ApiBearerAuth, ApiOperation } from '@nestjs/swagger';
import { JwtAuthGuard } from '@app/common/guards/jwt.auth.guard';
import { EnableDisableStatusEnum } from '@app/common/constants/days.enum';
import { BaseResponseDto } from '@app/common/dto/base.response.dto';
import { ControllerRoute } from '@app/common/constants/controller-route';
import { ProductService } from '../services';
import { Permissions } from 'src/decorators/permissions.decorator';
import { PermissionsGuard } from 'src/guards/permissions.guard';
import { AddProductDto } from '../dtos';

@ApiTags('Product Module')
@Controller({
  version: EnableDisableStatusEnum.ENABLED,
  path: ControllerRoute.PRODUCT.ROUTE,
})
export class ProductController {
  constructor(private readonly productService: ProductService) {}
  @ApiBearerAuth()
  @UseGuards(PermissionsGuard)
  @Permissions('PRODUCT_ADD')
  @Post()
  @ApiOperation({
    summary: ControllerRoute.PRODUCT.ACTIONS.CREATE_PRODUCT_SUMMARY,
    description: ControllerRoute.PRODUCT.ACTIONS.CREATE_PRODUCT_DESCRIPTION,
  })
  async addNewProductType(
    @Body() addProductDto: AddProductDto,
  ): Promise<BaseResponseDto> {
    return await this.productService.addNewProductType(addProductDto);
  }
  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Get()
  @ApiOperation({
    summary: ControllerRoute.PRODUCT.ACTIONS.LIST_PRODUCT_SUMMARY,
    description: ControllerRoute.PRODUCT.ACTIONS.LIST_PRODUCT_DESCRIPTION,
  })
  async getProducts(): Promise<BaseResponseDto> {
    return await this.productService.list();
  }
}
