import { Controller, Get, HttpStatus, UseGuards } from '@nestjs/common';
import { ApiBearerAuth, ApiOperation, ApiTags } from '@nestjs/swagger';
import { RoleService } from '../services/role.service';
import { EnableDisableStatusEnum } from '@app/common/constants/days.enum';
import { ControllerRoute } from '@app/common/constants/controller-route'; // Assuming this is where the routes are defined
import { JwtAuthGuard } from '@app/common/guards/jwt.auth.guard';

@ApiTags('Role Module')
@Controller({
  version: EnableDisableStatusEnum.ENABLED,
  path: ControllerRoute.ROLE.ROUTE, // use the static route constant
})
export class RoleController {
  constructor(private readonly roleService: RoleService) {}

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Get('types')
  @ApiOperation({
    summary: ControllerRoute.ROLE.ACTIONS.FETCH_ROLE_TYPES_SUMMARY,
    description: ControllerRoute.ROLE.ACTIONS.FETCH_ROLE_TYPES_DESCRIPTION,
  })
  async fetchRoleTypes() {
    const roleTypes = await this.roleService.fetchRoleTypes();
    return {
      statusCode: HttpStatus.OK,
      message: 'Role Types fetched Successfully',
      data: roleTypes,
    };
  }
}
