import { IsArray, ArrayNotEmpty, IsUUID, IsBoolean } from 'class-validator';
import { ApiProperty } from '@nestjs/swagger';

export class LinkSpacesToModelDto {
  @ApiProperty({
    description: 'List of space UUIDs to be linked to the space model',
    type: [String],
    example: [
      '550e8400-e29b-41d4-a716-446655440000',
      'f47ac10b-58cc-4372-a567-0e02b2c3d479',
    ],
  })
  @IsArray()
  @ArrayNotEmpty()
  @IsUUID('4', { each: true })
  spaceUuids: string[];

  @ApiProperty({
    description: 'Whether to overwrite existing space model links',
    type: Boolean,
    example: false,
  })
  @IsBoolean()
  overwrite: boolean;
}
