import { ModifyAction } from '@app/common/constants/modify-action.enum';
import {
  SpaceModelEntity,
  SpaceModelProductAllocationEntity,
  SpaceModelProductAllocationRepoitory,
  SubspaceModelEntity,
  SubspaceModelProductAllocationEntity,
  SubspaceModelProductAllocationRepoitory,
} from '@app/common/modules/space-model';
import { NewTagEntity } from '@app/common/modules/tag';
import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { ISingleSubspaceModel } from 'src/space-model/interfaces';
import { IUpdatedAllocations } from 'src/space-model/interfaces/subspace-product-allocation-update-result.interface';
import { ModifyTagDto } from 'src/space/dtos/tag/modify-tag.dto';
import { TagService as NewTagService } from 'src/tags/services';
import { In, QueryRunner } from 'typeorm';

@Injectable()
export class SubspaceModelProductAllocationService {
  constructor(
    private readonly tagService: NewTagService,
    private readonly subspaceModelProductAllocationRepository: SubspaceModelProductAllocationRepoitory,
    private readonly spaceModelAllocationRepository: SpaceModelProductAllocationRepoitory,
  ) {}

  async createProductAllocations(
    subspaceModel: SubspaceModelEntity,
    spaceModel: SpaceModelEntity,
    tags: NewTagEntity[],
    queryRunner?: QueryRunner,
    spaceAllocationsToExclude?: SpaceModelProductAllocationEntity[],
  ): Promise<IUpdatedAllocations[]> {
    // try {
    const updatedAllocations: IUpdatedAllocations[] = [];
    return updatedAllocations;
    //   const allocations: SubspaceModelProductAllocationEntity[] = [];
    //   for (const tag of tags) {
    //     // Step 1: Check if this specific tag is already allocated at the space level
    //     const existingTagInSpaceModel = await (queryRunner
    //       ? queryRunner.manager.findOne(SpaceModelProductAllocationEntity, {
    //           where: {
    //             spaceModel: {
    //               uuid: spaceModel.uuid,
    //             }, // Check at the space level
    //             tags: { uuid: tag.uuid }, // Check for the specific tag
    //           },
    //           relations: ['tags'],
    //         })
    //       : this.spaceModelAllocationRepository.findOne({
    //           where: {
    //             spaceModel: {
    //               uuid: spaceModel.uuid,
    //             },
    //             tags: { uuid: tag.uuid },
    //           },
    //           relations: ['tags', 'product'],
    //         }));
    //     const isExcluded = spaceAllocationsToExclude?.some(
    //       (excludedAllocation) =>
    //         excludedAllocation.product.uuid === tag.product.uuid &&
    //         excludedAllocation.tags.some((t) => t.uuid === tag.uuid),
    //     );
    //     // If tag is found at the space level, prevent allocation at the subspace level
    //     if (!isExcluded && existingTagInSpaceModel) {
    //       throw new HttpException(
    //         `Tag ${tag.uuid} (Product: ${tag.product.uuid}) is already allocated at the space level (${subspaceModel.spaceModel.uuid}). Cannot allocate the same tag in a subspace.`,
    //         HttpStatus.BAD_REQUEST,
    //       );
    //     }
    //     // Check if this specific tag is already allocated within another subspace of the same space
    //     const existingTagInSameSpace = await (queryRunner
    //       ? queryRunner.manager.findOne(SubspaceModelProductAllocationEntity, {
    //           where: {
    //             product: { uuid: tag.product.uuid },
    //             subspaceModel: { spaceModel: subspaceModel.spaceModel },
    //             tags: { uuid: tag.uuid }, // Ensure the exact tag is checked
    //           },
    //           relations: ['subspaceModel', 'tags'],
    //         })
    //       : this.subspaceModelProductAllocationRepository.findOne({
    //           where: {
    //             product: { uuid: tag.product.uuid },
    //             subspaceModel: { spaceModel: subspaceModel.spaceModel },
    //             tags: { uuid: tag.uuid },
    //           },
    //           relations: ['subspaceModel', 'tags'],
    //         }));
    //     // Prevent duplicate allocation if tag exists in another subspace of the same space
    //     if (
    //       existingTagInSameSpace &&
    //       existingTagInSameSpace.subspaceModel.uuid !== subspaceModel.uuid
    //     ) {
    //       throw new HttpException(
    //         `Tag ${tag.uuid} (Product: ${tag.product.uuid}) is already allocated in another subspace (${existingTagInSameSpace.subspaceModel.uuid}) within the same space (${subspaceModel.spaceModel.uuid}).`,
    //         HttpStatus.BAD_REQUEST,
    //       );
    //     }
    //     //Check if there are existing allocations for this product in the subspace
    //     const existingAllocationsForProduct = await (queryRunner
    //       ? queryRunner.manager.find(SubspaceModelProductAllocationEntity, {
    //           where: {
    //             subspaceModel: { uuid: subspaceModel.uuid },
    //             product: { uuid: tag.product.uuid },
    //           },
    //           relations: ['tags'],
    //         })
    //       : this.subspaceModelProductAllocationRepository.find({
    //           where: {
    //             subspaceModel: { uuid: subspaceModel.uuid },
    //             product: { uuid: tag.product.uuid },
    //           },
    //           relations: ['tags'],
    //         }));
    //     //Flatten all existing tags for this product in the subspace
    //     const existingTagsForProduct = existingAllocationsForProduct.flatMap(
    //       (allocation) => allocation.tags,
    //     );
    //     // Check if the tag is already assigned to the same product in this subspace
    //     const isDuplicateTag = existingTagsForProduct.some(
    //       (existingTag) => existingTag.uuid === tag.uuid,
    //     );
    //     if (isDuplicateTag) {
    //       throw new HttpException(
    //         `Tag ${tag.uuid} is already allocated to product ${tag.product.uuid} within this subspace (${subspaceModel.uuid}).`,
    //         HttpStatus.BAD_REQUEST,
    //       );
    //     }
    //     //  If no existing allocation, create a new one
    //     if (existingAllocationsForProduct.length === 0) {
    //       const allocation = queryRunner
    //         ? queryRunner.manager.create(SubspaceModelProductAllocationEntity, {
    //             subspaceModel,
    //             product: tag.product,
    //             tags: [tag],
    //           })
    //         : this.subspaceModelProductAllocationRepository.create({
    //             subspaceModel,
    //             product: tag.product,
    //             tags: [tag],
    //           });
    //       allocations.push(allocation);
    //     } else {
    //       //If allocation exists, add the tag to it
    //       existingAllocationsForProduct[0].tags.push(tag);
    //       if (queryRunner) {
    //         await queryRunner.manager.save(
    //           SubspaceModelProductAllocationEntity,
    //           existingAllocationsForProduct[0],
    //         );
    //       } else {
    //         await this.subspaceModelProductAllocationRepository.save(
    //           existingAllocationsForProduct[0],
    //         );
    //       }
    //       updatedAllocations.push({
    //         allocation: existingAllocationsForProduct[0],
    //         tagsAdded: [tag],
    //       });
    //     }
    //   }
    //   // Save newly created allocations
    //   if (allocations.length > 0) {
    //     if (queryRunner) {
    //       await queryRunner.manager.save(
    //         SubspaceModelProductAllocationEntity,
    //         allocations,
    //       );
    //     } else {
    //       await this.subspaceModelProductAllocationRepository.save(allocations);
    //     }
    //     allocations.forEach((allocation) => {
    //       updatedAllocations.push({ newAllocation: allocation });
    //     });
    //   }
    //   return updatedAllocations;
    // } catch (error) {
    //   throw new HttpException(
    //     `An unexpected error occurred while creating subspace product allocations ${error}`,
    //     HttpStatus.INTERNAL_SERVER_ERROR,
    //   );
    // }
  }

  async processDeleteActions(
    dtos: ModifyTagDto[],
    queryRunner: QueryRunner,
  ): Promise<SubspaceModelProductAllocationEntity[]> {
    try {
      if (!dtos || dtos.length === 0) {
        throw new Error('No DTOs provided for deletion.');
      }

      const tagUuidsToDelete = dtos
        // .filter((dto) => dto.action === ModifyAction.DELETE && dto.tagUuid)
        .map((dto) => dto.tagUuid);

      if (tagUuidsToDelete.length === 0) return [];

      const allocationsToUpdate = await queryRunner.manager.find(
        SubspaceModelProductAllocationEntity,
        {
          where: { tag: In(tagUuidsToDelete) },
        },
      );

      if (!allocationsToUpdate || allocationsToUpdate.length === 0) return [];

      const deletedAllocations: SubspaceModelProductAllocationEntity[] = [];
      const allocationUpdates: SubspaceModelProductAllocationEntity[] = [];

      for (const allocation of allocationsToUpdate) {
        // const updatedTags = allocation.tags.filter(
        //   (tag) => !tagUuidsToDelete.includes(tag.uuid),
        // );
        // if (updatedTags.length === allocation.tags.length) {
        //   continue;
        // }
        // if (updatedTags.length === 0) {
        //   deletedAllocations.push(allocation);
        // } else {
        //   allocation.tags = updatedTags;
        //   allocationUpdates.push(allocation);
        // }
      }

      if (allocationUpdates.length > 0) {
        await queryRunner.manager.save(
          SubspaceModelProductAllocationEntity,
          allocationUpdates,
        );
      }

      if (deletedAllocations.length > 0) {
        await queryRunner.manager.remove(
          SubspaceModelProductAllocationEntity,
          deletedAllocations,
        );
      }

      await queryRunner.manager
        .createQueryBuilder()
        .delete()
        .from('subspace_model_product_tags')
        .where(
          'subspace_model_product_allocation_uuid NOT IN (' +
            queryRunner.manager
              .createQueryBuilder()
              .select('allocation.uuid')
              .from(SubspaceModelProductAllocationEntity, 'allocation')
              .getQuery() +
            ')',
        )
        .execute();

      return deletedAllocations;
    } catch (error) {
      throw this.handleError(error, `Failed to delete tags in subspace model`);
    }
  }

  async updateAllocations(
    subspaceModels: ISingleSubspaceModel[],
    projectUuid: string,
    queryRunner: QueryRunner,
    spaceModel: SpaceModelEntity,
    spaceTagUpdateDtos?: ModifyTagDto[],
  ) {
    // : Promise<IUpdatedAllocations[]>
    const spaceAllocationToExclude: SpaceModelProductAllocationEntity[] = [];
    const updatedAllocations: IUpdatedAllocations[] = [];
    return updatedAllocations;
    // for (const subspaceModel of subspaceModels) {
    //   const tagDtos = subspaceModel.tags;
    //   if (tagDtos.length > 0) {
    //     const tagsToAddDto: ProcessTagDto[] = tagDtos
    //       .filter((dto) => dto.action === ModifyAction.ADD)
    //       .map((dto) => ({
    //         name: dto.name,
    //         productUuid: dto.productUuid,
    //         uuid: dto.newTagUuid,
    //       }));

    //     const tagsToDeleteDto = tagDtos.filter(
    //       (dto) => dto.action === ModifyAction.DELETE,
    //     );

    //     if (tagsToAddDto.length > 0) {
    //       let processedTags = await this.tagService.processTags(
    //         tagsToAddDto,
    //         projectUuid,
    //         queryRunner,
    //       );

    //       for (const subspaceDto of subspaceModels) {
    //         if (
    //           subspaceDto !== subspaceModel &&
    //           subspaceDto.action === ModifyAction.UPDATE &&
    //           subspaceDto.tags
    //         ) {
    //           // Tag is deleted from one subspace and added in another subspace
    //           const deletedTags = subspaceDto.tags.filter(
    //             (tagDto) =>
    //               tagDto.action === ModifyAction.DELETE &&
    //               processedTags.some((tag) => tag.uuid === tagDto.tagUuid),
    //           );

    //           for (const deletedTag of deletedTags) {
    //             const allocation = await queryRunner.manager.findOne(
    //               SubspaceModelProductAllocationEntity,
    //               {
    //                 where: {
    //                   tags: {
    //                     uuid: deletedTag.tagUuid,
    //                   },
    //                   subspaceModel: {
    //                     uuid: subspaceDto.subspaceModel.uuid,
    //                   },
    //                 },
    //                 relations: ['tags', 'product', 'subspaceModel'],
    //               },
    //             );
    //             if (allocation) {
    //               const isCommonTag = allocation.tags.some(
    //                 (tag) => tag.uuid === deletedTag.tagUuid,
    //               );

    //               if (allocation && isCommonTag) {
    //                 const tagEntity = allocation.tags.find(
    //                   (tag) => tag.uuid === deletedTag.tagUuid,
    //                 );

    //                 allocation.tags = allocation.tags.filter(
    //                   (tag) => tag.uuid !== deletedTag.tagUuid,
    //                 );

    //                 updatedAllocations.push({
    //                   allocation,
    //                   tagsRemoved: [tagEntity],
    //                 });

    //                 await queryRunner.manager.save(allocation);

    //                 const productAllocationExistInSubspace =
    //                   await queryRunner.manager.findOne(
    //                     SubspaceModelProductAllocationEntity,
    //                     {
    //                       where: {
    //                         subspaceModel: {
    //                           uuid: subspaceDto.subspaceModel.uuid,
    //                         },
    //                         product: { uuid: allocation.product.uuid },
    //                       },
    //                       relations: ['tags'],
    //                     },
    //                   );

    //                 if (productAllocationExistInSubspace) {
    //                   productAllocationExistInSubspace.tags.push(tagEntity);

    //                   updatedAllocations.push({
    //                     allocation: productAllocationExistInSubspace,
    //                     tagsAdded: [tagEntity],
    //                   });

    //                   await queryRunner.manager.save(
    //                     productAllocationExistInSubspace,
    //                   );
    //                 } else {
    //                   const newProductAllocation = queryRunner.manager.create(
    //                     SubspaceModelProductAllocationEntity,
    //                     {
    //                       subspaceModel: subspaceModel.subspaceModel,
    //                       product: allocation.product,
    //                       tags: [tagEntity],
    //                     },
    //                   );

    //                   updatedAllocations.push({
    //                     allocation: newProductAllocation,
    //                   });

    //                   await queryRunner.manager.save(newProductAllocation);
    //                 }

    //                 // Remove the tag from processedTags to prevent duplication
    //                 processedTags = processedTags.filter(
    //                   (tag) => tag.uuid !== deletedTag.tagUuid,
    //                 );

    //                 // Remove the tag from subspaceDto.tags to ensure it's not processed again while processing other dtos.
    //                 subspaceDto.tags = subspaceDto.tags.filter(
    //                   (tagDto) => tagDto.tagUuid !== deletedTag.tagUuid,
    //                 );
    //               }
    //             }
    //           }
    //         }
    //         if (
    //           subspaceDto !== subspaceModel &&
    //           subspaceDto.action === ModifyAction.DELETE
    //         ) {
    //           const allocation = await queryRunner.manager.findOne(
    //             SubspaceModelProductAllocationEntity,
    //             {
    //               where: {
    //                 subspaceModel: { uuid: subspaceDto.subspaceModel.uuid },
    //               },
    //               relations: ['tags'],
    //             },
    //           );

    //           const repeatedTags = allocation?.tags.filter((tag) =>
    //             processedTags.some(
    //               (processedTag) => processedTag.uuid === tag.uuid,
    //             ),
    //           );
    //           if (repeatedTags.length > 0) {
    //             allocation.tags = allocation.tags.filter(
    //               (tag) =>
    //                 !repeatedTags.some(
    //                   (repeatedTag) => repeatedTag.uuid === tag.uuid,
    //                 ),
    //             );

    //             updatedAllocations.push({
    //               allocation: allocation,
    //               tagsRemoved: repeatedTags,
    //             });

    //             await queryRunner.manager.save(allocation);

    //             const productAllocationExistInSubspace =
    //               await queryRunner.manager.findOne(
    //                 SubspaceModelProductAllocationEntity,
    //                 {
    //                   where: {
    //                     subspaceModel: { uuid: subspaceDto.subspaceModel.uuid },
    //                     product: { uuid: allocation.product.uuid },
    //                   },
    //                   relations: ['tags'],
    //                 },
    //               );

    //             if (productAllocationExistInSubspace) {
    //               updatedAllocations.push({
    //                 allocation: productAllocationExistInSubspace,
    //                 tagsAdded: repeatedTags,
    //               });

    //               productAllocationExistInSubspace.tags.push(...repeatedTags);
    //               await queryRunner.manager.save(
    //                 productAllocationExistInSubspace,
    //               );
    //             } else {
    //               const newProductAllocation = queryRunner.manager.create(
    //                 SubspaceModelProductAllocationEntity,
    //                 {
    //                   subspaceModel: subspaceModel.subspaceModel,
    //                   product: allocation.product,
    //                   tags: repeatedTags,
    //                 },
    //               );

    //               updatedAllocations.push({
    //                 newAllocation: newProductAllocation,
    //               });

    //               await queryRunner.manager.save(newProductAllocation);
    //             }
    //           }
    //         }
    //       }
    //       if (spaceTagUpdateDtos) {
    //         const deletedSpaceTags = spaceTagUpdateDtos.filter(
    //           (tagDto) =>
    //             tagDto.action === ModifyAction.DELETE &&
    //             processedTags.some((tag) => tag.uuid === tagDto.tagUuid),
    //         );
    //         for (const deletedTag of deletedSpaceTags) {
    //           const allocation = await queryRunner.manager.findOne(
    //             SpaceModelProductAllocationEntity,
    //             {
    //               where: {
    //                 spaceModel: { uuid: spaceModel.uuid },
    //                 tags: { uuid: deletedTag.tagUuid },
    //               },
    //               relations: ['tags', 'product'],
    //             },
    //           );

    //           if (
    //             allocation &&
    //             allocation.tags.some((tag) => tag.uuid === deletedTag.tagUuid)
    //           ) {
    //             spaceAllocationToExclude.push(allocation);
    //           }
    //         }
    //       }

    //       // Create new product allocations
    //       const newAllocations = await this.createProductAllocations(
    //         subspaceModel.subspaceModel,
    //         spaceModel,
    //         processedTags,
    //         queryRunner,
    //         spaceAllocationToExclude,
    //       );
    //       return [...updatedAllocations, ...newAllocations];
    //     }
    //     if (tagsToDeleteDto.length > 0) {
    //       await this.processDeleteActions(tagsToDeleteDto, queryRunner);
    //     }
    //   }
    // }
  }

  async clearAllAllocations(subspaceIds: string[], queryRunner: QueryRunner) {
    try {
      await queryRunner.manager.delete(SubspaceModelProductAllocationEntity, {
        subspaceModel: In(subspaceIds),
      });

      await queryRunner.manager
        .createQueryBuilder()
        .delete()
        .from(SubspaceModelProductAllocationEntity)
        .where('"subspace_model_uuid" IN (:...subspaceIds)', {
          subspaceIds,
        })
        .execute();
    } catch (error) {
      throw this.handleError(
        error,
        `Failed to clear all allocations subspace model product allocation`,
      );
    }
  }

  private handleError(error: any, message: string): HttpException {
    return new HttpException(
      error instanceof HttpException ? error.message : message,
      error instanceof HttpException
        ? error.getStatus()
        : HttpStatus.INTERNAL_SERVER_ERROR,
    );
  }
}
