import { ORPHAN_SPACE_NAME } from '@app/common/constants/orphan-constant';
import { ApiProperty, ApiPropertyOptional } from '@nestjs/swagger';
import { Type } from 'class-transformer';
import {
  IsArray,
  IsNumber,
  IsOptional,
  IsString,
  NotEquals,
  ValidateNested,
} from 'class-validator';
import { CreateProductAllocationDto } from './create-product-allocation.dto';
import { UpdateSubspaceDto } from './subspace';

export class UpdateSpaceDto {
  @ApiProperty({
    description: 'Updated name of the space ',
    example: 'New Space Name',
  })
  @IsOptional()
  @IsString()
  @NotEquals(ORPHAN_SPACE_NAME, {
    message() {
      return `Space name cannot be "${ORPHAN_SPACE_NAME}". Please choose a different name.`;
    },
  })
  spaceName?: string;

  @ApiProperty({
    description: 'Icon identifier for the space',
    example: 'assets/location',
    required: false,
  })
  @IsString()
  @IsOptional()
  public icon?: string;

  @ApiProperty({ description: 'X position on canvas', example: 120 })
  @IsNumber()
  @IsOptional()
  x?: number;

  @ApiProperty({ description: 'Y position on canvas', example: 200 })
  @IsNumber()
  @IsOptional()
  y?: number;

  @ApiPropertyOptional({
    description: 'List of subspace modifications',
    type: [UpdateSubspaceDto],
  })
  @IsOptional()
  @IsArray()
  @ValidateNested({ each: true })
  @Type(() => UpdateSubspaceDto)
  subspaces?: UpdateSubspaceDto[];

  @ApiPropertyOptional({
    description: 'List of allocations modifications',
    type: [CreateProductAllocationDto],
  })
  @IsOptional()
  @IsArray()
  @ValidateNested({ each: true })
  @Type(() => CreateProductAllocationDto)
  productAllocations?: CreateProductAllocationDto[];

  @ApiProperty({
    description: 'UUID of the Space',
    example: 'd290f1ee-6c54-4b01-90e6-d701748f0851',
  })
  @IsString()
  @IsOptional()
  spaceModelUuid?: string;
}
