import { Controller, Get, HttpStatus } from '@nestjs/common';
import { ApiOperation, ApiTags } from '@nestjs/swagger';
import { EnableDisableStatusEnum } from '@app/common/constants/days.enum';
import { ControllerRoute } from '@app/common/constants/controller-route'; // Assuming this is where the routes are defined
import { TermsAndConditionsService } from '../services';

@ApiTags('Terms & Conditions Module')
@Controller({
  version: EnableDisableStatusEnum.ENABLED,
  path: ControllerRoute.TERMS_AND_CONDITIONS.ROUTE, // use the static route constant
})
export class TermsConditionsController {
  constructor(
    private readonly termsAndConditionsService: TermsAndConditionsService,
  ) {}

  @Get()
  @ApiOperation({
    summary: ControllerRoute.TERMS_AND_CONDITIONS.ACTIONS.FETCH_TERMS_SUMMARY,
    description:
      ControllerRoute.TERMS_AND_CONDITIONS.ACTIONS.FETCH_TERMS_DESCRIPTION,
  })
  async fetchTermsAndConditions() {
    const htmlContent =
      await this.termsAndConditionsService.fetchTermsAndConditions();
    return {
      statusCode: HttpStatus.OK,
      message: 'Terms and conditions fetched successfully',
      data: htmlContent,
    };
  }
}
